# -*- coding:Utf-8 -*-
import es, re, psyco, dm_python
psyco.full()


admin = set()
command = set()
randguns = set()
importname = __import__(__name__)
regex = re.compile('//.*|(\r|\n|")+')
cfgpath = es.getString("eventscripts_gamedir") + "/cfg/dm_python/"

class DmConfig(object):
    """ Config Management """
    def __init__(self):
        # config.cfg
        self.steamid = es.ServerVar("dp_admin_steamid", "", "[dm_python] administrator steamid")
        self.admin = es.ServerVar("dp_admin_command", "", "[dm_python] administration menu command")
        self.command = es.ServerVar("dp_menu_command", "", "[dm_python] deathmatch menu command")
        self.randguns = es.ServerVar("dp_randguns_command", "", "[dm_python] random weapons command")
        self.style = es.ServerVar("dp_menu_style", "", "[dm_python] deathmatch menu style")
        self.timeout = es.ServerVar("dp_menu_timeout", "", "[dm_python] menu time out")
        self.dissolver = es.ServerVar("dp_ragdoll_dissolver", "", "[dm_python] dissolve ragdoll")

        # respawn.cfg
        self.plugin = es.ServerVar("dp_respawn_plugin", "", "[dm_python] plugin used for respawn players")
        self.clicspawn = es.ServerVar("dp_respawn_clic", "", "[dm_python] time for allow a player to spawn himself")
        self.respawn = es.ServerVar("dp_respawn_time", "", "[dm_python] time for respawn a player after his dead")
        self.random = es.ServerVar("dp_respawn_random", "", "[dm_python] respawn player at his team spawn")
        self.noblock = es.ServerVar("dp_respawn_noblock", "", "[dm_python] no block method (0|1|2)")
        self.protection = es.ServerVar("dp_respawn_protection", "", "[dm_python] protect player when he spawn")
        self.alpha = es.ServerVar("dp_respawn_alpha", "", "[dm_python] color of player when he's protected")

        # items.cfg
        self.assaultsuit = es.ServerVar("dp_give_assaultsuit", "", "[dm_python] give assaultsuit")
        self.nightvision = es.ServerVar("dp_give_nightvision", "", "[dm_python] give nightvision")
        self.hegrenade = es.ServerVar("dp_hegrenade_max", "", "[dm_python] hegrenade max")
        self.flashbang = es.ServerVar("dp_flashbang_max", "", "[dm_python] flashbang max")
        self.smokegrenade = es.ServerVar("dp_smokegrenade_max", "", "[dm_python] smokegrenade max")

        # bots.cfg
        self.botRespawn = es.ServerVar("dp_bot_respawn", "", "[dm_python] time to force bot to respawn after his death")
        self.botPrimary = es.ServerVar("dp_bot_primary", "", "[dm_python] primary weapon allowed for bot")
        self.botSecondary = es.ServerVar("dp_bot_secondary", "", "[dm_python] secondary weapon allowed for bot")

        # votemap.cfg
        self.kills = es.ServerVar("dp_vote_kills", "", "[dm_python] activate votemap")
        self.time = es.ServerVar("dp_vote_time", "", "[dm_python] time for vote")
        self.maxmap = es.ServerVar("dp_vote_maxmap", "", "[dm_python] number of map added in vote")
        self.lastmaps = es.ServerVar("dp_vote_lastmaps", "", "[dm_python] don't add last maps played in vote")
        self.startsound = es.ServerVar("dp_vote_startsound", "", "[dm_python] sound played at vote begening")
        self.endsound = es.ServerVar("dp_vote_endsound", "", "[dm_python] sound played when vote is ending")

        # extension.cfg
        self.extension = es.ServerVar("dp_extension_enable_script", "", "[dm_python] extended configuration")
        self.reloadclip = es.ServerVar("dp_extension_reload_clip", "", "[dm_python] reload clip when player make a kill")
        self.giveammo = es.ServerVar("dp_extension_give_ammo", "", "[dm_python] give ammo when munitions are emptie")

        # store convars and associated informations (instance, type of value, name of cfg file)
        self.convar = {
            "admin" : (self.admin, None, "config"),
            "command" : (self.command, None, "config"),
            "randguns" : (self.randguns, None, "config"),
            "style" : (self.style, ("0", "1", "2"), "config"),
            "timeout" : (self.timeout, 0, "config"),
            "dissolver": (self.dissolver, ("0", "1"), "config"),

            "clicspawn" : (self.clicspawn, 1., "respawn"),
            "respawn" : (self.respawn, 1., "respawn"),
            "random" : (self.random, ("0", "1"), "respawn"),
            "noblock" : (self.noblock, ("0", "1", "2"), "respawn"),
            "protection" : (self.protection, 0., "respawn"),
            "alpha" : (self.alpha, str, "respawn"),

            "assaultsuit" : (self.assaultsuit, ("0", "1"), "items"),
            "nightvision" : (self.nightvision, ("0", "1"), "items"),
            "hegrenade" : (self.hegrenade, 0, "items"),
            "flashbang" : (self.flashbang, 0, "items"),
            "smokegrenade" : (self.smokegrenade, 0, "items"),

            "kills" : (self.kills, 0, "votemap"),
            "time" : (self.time, 0, "votemap"),
            "maxmap" : (self.maxmap, 0, "votemap"),
            "lastmaps" : (self.lastmaps, 0, "votemap"),

            "extension" : (self.extension, ("0", "1"), "extension"),
            "reloadclip" : (self.reloadclip, ("0", "1", "2"), "extension"),
            "giveammo" : (self.giveammo, ("0", "1"), "extension")}
            

    def __contains__(self, item):
        # return True if the steamid is an admin
        return str(item) in map(lambda steamid: steamid.strip(), str(self.steamid).split(","))

    def __getitem__(self, item):
        # get ServerVar instance of cvars, type of value, supported value, name of the CFG
        if item == "plugin":
            plugin = ["es"]
            if dm_python.SourcePythonExtensions: plugin.append("spe")
            if dm_python.NativeTools: plugin.append("nt")
            return (self.plugin, tuple(plugin), "config")
        else: return self.convar[item]

    def update(self):
        # execute config and create command
        self.execCfg()
        self.createCmd()

    def execCfg(self):
        # execute config
        es.mexec("dm_python/bots.cfg")
        es.mexec("dm_python/items.cfg")
        es.mexec("dm_python/config.cfg")
        es.mexec("dm_python/respawn.cfg")
        es.mexec("dm_python/extension.cfg")
        es.mexec("dm_python/vote/votemap.cfg")

        # check if SPE can be used if it's selected
        if str(self.plugin) == "spe" and not dm_python.SourcePythonExtensions:
            self.plugin.set("es")
            self.saveCfg("config")
            es.dbgmsg(0, 'dm_python: cannot use plugin Source Python Extensions, "dp_respawn_plugin" has been modified to "es"')

        # check if NT can be used if it's selected
        elif str(self.plugin) == "nt" and not dm_python.NativeTools:
            self.plugin.set("es")
            self.saveCfg("config")
            es.dbgmsg(0, 'dm_python: cannot use plugin Native Tools, "dp_respawn_plugin" has been modified to "es"')

        # un/load dm_extension if it has to be
        if int(self.extension):
            if not dm_python.extensionLoaded(): es.load("dm_python/dm_extension")
        else:
            if dm_python.extensionLoaded(): es.unload("dm_python/dm_extension")

        # download vote sounds
        startsound, endsound = str(self.startsound), str(self.endsound)
        if startsound: es.stringtable("downloadables", "sound/" + startsound)
        if endsound: es.stringtable("downloadables", "sound/" + endsound)

    def saveCfg(self, save = "all"):
        # create and save all config files
        if save == "all":
            self.saveCfg("config")
            self.saveCfg("respawn")
            self.saveCfg("items")
            self.saveCfg("votemap")
            self.saveCfg("extension")

        # ../cfg/dm_python/config.cfg
        elif save == "config":
            try:
                openfile = open(cfgpath + "config.cfg", "w")
                openfile.write(
                    "// config.cfg for dm_python script\n\n\n" +
                    '// X = administrator steamid\ndp_admin_steamid "%s"\n\n' % self.steamid +
                    '// X = administration menu command\ndp_admin_command "%s"\n\n' % self.admin +
                    '// X = deathmatch menu command\ndp_menu_command "%s"\n\n' % self.command +
                    '// X = command for un/activate random weapons\ndp_randguns_command "%s"\n\n' % self.randguns +
                    '// X = deathmatch menu style (1|2| 0 = disable menu)\ndp_menu_style "%s"\n\n' % self.style +
                    '// X = close menu after X second, 0 = do not close automatically\ndp_menu_timeout "%s"\n\n' % self.timeout +
                    '// 0 = no effect at ragdoll removing, 1 = dissolve effect\ndp_ragdoll_dissolver "%s"' % self.dissolver)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot acces to the file cfg/dm_python/config.cfg")

        # ../cfg/dm_python/respawn.cfg
        elif save == "respawn":
            try:
                openfile = open(cfgpath + "respawn.cfg", "w")
                openfile.write(
                    "// respawn.cfg for dm_python script\n\n\n" +
                    '// X = plugin used for respawn a player, "es" (EventScripts), "spe" (SourcePythonExtensions) or "nt" (NativeTools)\ndp_respawn_plugin "%s"\n\n' % self.plugin +
                    '// X = time for allow a player to spawn himself (+attack) after his death (1 second minimum), 0 = desactivate\ndp_respawn_clic "%s"\n\n' % self.clicspawn +
                    '// X = time to force player to respawn after his death (1 second minimum)\ndp_respawn_time "%s"\n\n' % self.respawn +
                    '// 0 = respawn player at his team spawn, 1 = respawn player at any spawn team\ndp_respawn_random "%s"\n\n' % self.random +
                    '// 0 = disable noblock, 1 = enable every time, 2 = player become solide when he is alone\ndp_respawn_noblock "%s"\n\n' % self.noblock +
                    '// X = time of the protection (in second), 0 = disable protection\ndp_respawn_protection "%s"\n\n' % self.protection +
                    '// 0~255 = alpha of player when he\'s protected\ndp_respawn_alpha "%s"' % self.alpha)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot acces to the file cfg/dm_python/respawn.cfg")

        # ../cfg/dm_python/items.cfg
        elif save == "items":
            try:
                openfile = open(cfgpath + "items.cfg", "w")
                openfile.write(
                    "// items.cfg for dm_python script\n\n\n" +
                    '// 1 = give kevlar and helmet\ndp_give_assaultsuit "%s"\n\n' % self.assaultsuit +
                    '// 1 = give nightvision\ndp_give_nightvision "%s"\n\n' % self.nightvision +
                    '// X = hegrenade max\ndp_hegrenade_max "%s"\n\n' % self.hegrenade +
                    '// X = flashbang max\ndp_flashbang_max "%s"\n\n' % self.flashbang +
                    '// X = smokegrenade max\ndp_smokegrenade_max "%s"\n\n' % self.smokegrenade)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot acces to the file cfg/dm_python/items.cfg")

        # ../cfg/dm_python/items.cfg
        elif save == "bots":
            try:
                openfile = open(cfgpath + "bots.cfg", "w")
                openfile.write(
                    "// items.cfg for dm_python script\n\n\n" +
                    '// 1 = time to force the bot to respawn after his death (1 second minimum)\ndp_bot_respawn "%s"\n\n' % self.botRespawn +
                    '// X = allowed primary weapons for bots, "" = all, 0 = no primary weapon\ndp_bot_primary "%s"\n\n' % self.botPrimary +
                    '// X = allowed secondary weapons for bots, "" = all, 0 = no secondary weapon\ndp_bot_secondary "%s"' % self.botSecondary)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot acces to the file cfg/dm_python/bots.cfg")

        # ../cfg/dm_python/vote/votemap.cfg
        elif save == "votemap":
            try:
                openfile = open(cfgpath + "vote/votemap.cfg", "w")
                openfile.write(
                    "// votemap.cfg for dm_python script\n\n\n" +
                    '// X = submit votemap after X kill, 0 = disable votemap\ndp_vote_kills "%s"\n\n' % self.kills +
                    '// X = time for vote (sec)\ndp_vote_time "%s"\n\n' % self.time +
                    '// X = number of map added into the vote\ndp_vote_maxmap "%s"\n\n' % self.maxmap +
                    '// X = don\'t add the X last maps played into the vote\ndp_vote_lastmaps "%s"\n\n' % self.lastmaps +
                    '// X = sound played at vote beginning\ndp_vote_startsound "%s"\n\n' % self.startsound +
                    '// X = sound played when vote is ended\ndp_vote_endsound "%s"' % self.endsound)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot acces to the file cfg/dm_python/vote/votemap.cfg")

        # ../cfg/dm_python/extension.cfg
        elif save == "extension":
            try:
                openfile = open(cfgpath + "extension.cfg", "w")
                openfile.write(
                    "// extension.cfg for dm_python script\n\n\n" +
                    '// 0 = disable extended configuration, 1 = enable\ndp_extension_enable_script "%s"\n\n' % self.extension +
                    '// 2 = reload clip when player make a kill, 1 = only by headshot, 0 = off\ndp_extension_reload_clip "%s"\n\n' % self.reloadclip +
                    '// 1 = give ammo when munitions are emptie, 0 = off\ndp_extension_give_ammo "%s"' % self.giveammo)
                openfile.close()
            except IOError: es.dbgmsg(0, "dm_python: cannot acces to the file cfg/dm_python/votemap.cfg")

    def createCmd(self, cmd = "all"):
        # create all script commands
        if cmd == "all":
            self.createCmd("admin")
            self.createCmd("command")
            self.createCmd("randguns")

        # admin menu
        elif cmd == "admin":
            self.deleteCmd("admin")
            for word in map(lambda word: word.strip(), str(self.admin).split(",")):
                admin.add(word)
                es.regsaycmd(word, "dm_python/administrationMenu", "[dm_python] administration menu acces")
                es.regclientcmd(word, "dm_python/administrationMenu", "[dm_python] administration menu acces")

        # weapon menu
        elif cmd == "command":
            self.deleteCmd("command")
            for word in map(lambda word: word.strip(), str(self.command).split(",")):
                command.add(word)
                es.regsaycmd(word, "dm_python/deathmatchMenu", "[dm_python] deathmatch menu acces")
                es.regclientcmd(word, "dm_python/deathmatchMenu", "[dm_python] deathmatch menu acces")

        # random weapons
        elif cmd == "randguns":
            self.deleteCmd("randguns")
            for word in map(lambda word: word.strip(), str(self.randguns).split(",")):
                randguns.add(word)
                es.regsaycmd(word, "dm_python/randomWeapons", "[dm_python] random weapons command")
                es.regclientcmd(word, "dm_python/randomWeapons", "[dm_python] random weapons command")

    def deleteCmd(self, cmd = "all"):
        # delete and clear all script commands
        if cmd == "all":
            self.deleteCmd("admin")
            self.deleteCmd("command")
            self.deleteCmd("randguns")

        # admin menu
        elif cmd == "admin":
            for word in admin:
                es.unregsaycmd(word)
                es.unregclientcmd(word)
            admin.clear()

        # weapon menu
        elif cmd == "command":
            for word in command:
                es.unregsaycmd(word)
                es.unregclientcmd(word)
            command.clear()

        # random weapons
        elif cmd == "randguns":
            for word in randguns:
                es.unregsaycmd(word)
                es.unregclientcmd(word)
            randguns.clear()

# create DmConfig object
dmConfig = DmConfig()
