# -*- coding:Utf-8 -*-
import es, psyco, random, cPickle, playerlib, effectlib, gamethread, dm_player, dm_entities
from dm_config import dmConfig
psyco.full()


importname = __import__(__name__)
currentmap = es.ServerVar("eventscripts_currentmap")
spawnfile = es.getString("eventscripts_gamedir") + "/addons/eventscripts/dm_python/spawnpoints.ini"

class DmSpawns(object):
    """ Spawns Management """

    def __init__(self):
        self.spawns = {}
        self.terrorist = []
        self.counterterrorist = []
        self.isupdated = False

        # try to load spawnpoints
        try:
            openfile = open(spawnfile, "rb")
            self.spawns = cPickle.load(openfile)
            openfile.close()
        # create the file if it does not exists
        except IOError: self.save()

    def i_search(self):
        # check if spawnpoints exist for this map
        mapname = str(currentmap)
        if not self.spawns.has_key(mapname):
            self.spawns[mapname] = ([], [])
            self.i_store(mapname, 2)
            self.i_store(mapname, 3)
        self.terrorist, self.counterterrorist = self.spawns[mapname]
        self.isupdated = True

    def i_store(self, mapname, team):
        self.i_delete(mapname, team)
        if team == 2: spawns, info = self.spawns[mapname][0], "info_player_terrorist"
        else: spawns, info = self.spawns[mapname][1], "info_player_counterterrorist"
        for index in es.createentitylist(info): spawns.insert(
            0, map(float, es.getindexprop(index, "CBaseEntity.m_vecOrigin").split(",")) + map(float, es.getindexprop(index, "CBaseEntity.m_angRotation").split(",")) + [True])

    def i_delete(self, mapname, team, type = True):
        spawns = self.terrorist if team == 2 else self.counterterrorist
        for spawn in spawns[:]:
            if spawn[6] == type: spawns.remove(spawn)

    def i_show(self, spawns, current, color):
        try:
            ox, oy, oz, ax, ay, az, origin = spawns[current]
            self.i_effect([ox, oy, oz], color)
            gamethread.delayedname(0.001, "_dm_python_show_spawnpoints", self.i_show, (spawns, current + 1, color))
        except IndexError: pass

    def i_effect(self, coord1, color):
        coord2 = coord1[:]
        coord1[0] -= 30
        coord1[1] += 30
        coord2[0] += 30
        coord2[1] -= 30
        coord2[2] += 100
        if color == "red": effectlib.drawBox(coord1, coord2, seconds = 2, width = 6, endwidth = 6, green = 0, blue = 0)
        elif color == "blue": effectlib.drawBox(coord1, coord2, seconds = 2, width = 6, endwidth = 6, green = 0, red = 0)
        elif color == "green": effectlib.drawBox(coord1, coord2, seconds = 2, width = 6, endwidth = 6, red = 0, blue = 0)
        elif color == "white": effectlib.drawBox(coord1, coord2, seconds = 2, width = 6, endwidth = 6)

    def search(self, delayed = False):
        if delayed: gamethread.delayed(2, self.i_search)
        else: self.i_search()

    def save(self):
        # save self.spawns object (spawnpoints)
        try:
            openfile = open(spawnfile, "wb")
            cPickle.dump(self.spawns, openfile)
            openfile.close()
        except IOError: es.dbgmsg(0, "dm_python: cannot save file addons/eventscripts/dm_python/spawnpoints.ini")

    def add(self, userid):
        user = playerlib.getPlayer(userid)
        ax, ay, az = user.viewangle
        ox, oy, oz = user.location

        if user.teamid == 2: spawns, color = self.terrorist, "red"
        else: spawns, color = self.counterterrorist, "blue"
        spawns.append((ox, oy, oz, ax, ay, az, False))
        self.i_effect([ox, oy, oz], color)

    def sub(self, userid):
        spawns = self.terrorist if es.getplayerteam(userid) == 2 else self.counterterrorist
        index = dm_player.player(userid).indexspawn
        ox, oy, oz, ax, ay, az, origin = spawns[index]
        self.i_effect([ox, oy, oz], "white")
        del spawns[index]

    def set(self, userid):
        try:
            spawns = self.terrorist if es.getplayerteam(userid) == 2 else self.counterterrorist
            ox, oy, oz, ax, ay, az, origin = spawns[dm_player.player(userid).indexspawn]
            dm_entities.setPositionOfPlayer(userid, ox, oy, oz, ax, ay, az)
            self.i_effect([ox, oy, oz], "green")
        except IndexError: pass

    def get(self, userid):
        if int(dmConfig.random): spawns = self.terrorist + self.counterterrorist
        else: spawns = self.terrorist if es.getplayerteam(userid) == 2 else self.counterterrorist
        return random.choice(spawns) if spawns else None

    def show(self, userid, type = "all", color = None):
        spawns = self.getSpawnsList(userid, type)
        gamethread.cancelDelayed("_dm_python_show_spawnpoints")
        if color is None: color = "red" if es.getplayerteam(userid) == 2 else "blue"
        self.i_show(spawns, 0, color)

    def restore(self, userid):
        self.i_store(str(currentmap), es.getplayerteam(userid))
        self.show(userid, "origin", "green")

    def delete(self, userid, type = "all"):
        spawns = self.getSpawnsList(userid, type)
        gamethread.cancelDelayed("_dm_python_show_spawnpoints")
        self.show(userid, type, "white")

        if type == "origin": self.i_delete(str(currentmap), es.getplayerteam(userid), True)
        elif type == "custom": self.i_delete(str(currentmap), es.getplayerteam(userid), False)
        elif type == "all":
            team = es.getplayerteam(userid)
            self.i_delete(str(currentmap), team, True)
            self.i_delete(str(currentmap), team, False)

    def getSpawnsList(self, userid, type = "all"):
        if type == "all": return self.terrorist[:] if es.getplayerteam(userid) == 2 else self.counterterrorist[:]
        elif type == "origin": return [spawn for spawn in self.terrorist if spawn[6]] if es.getplayerteam(userid) == 2 else [spawn for spawn in self.counterterrorist if spawn[6]]
        elif type == "custom": return [spawn for spawn in self.terrorist if not spawn[6]] if es.getplayerteam(userid) == 2 else [spawn for spawn in self.counterterrorist if not spawn[6]]

# create DmSpawns object
dmSpawns = DmSpawns()
